@@ -1,18 +1,28 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP                #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 
 -- | Internal types and functions.
 
 module Path.Internal
-  (Path(..))
+  ( Path(..)
+  , hasParentDir
+  )
   where
 
 import Control.DeepSeq (NFData (..))
 import Data.Aeson (ToJSON (..))
 import Data.Data
+import Data.Hashable
+import Data.List
+import qualified System.FilePath as FilePath
 
 -- | Path of some base and type.
 --
+-- The type variables are:
+--
+--   * @b@ — base, the base location of the path; absolute or relative.
+--   * @t@ — type, whether file or directory.
+--
 -- Internally is a string. The string can be of two formats only:
 --
 -- 1. File format: @file.txt@, @foo\/bar.txt@, @\/foo\/bar.txt@
@@ -39,7 +49,7 @@ instance Eq (Path b t) where
 instance Ord (Path b t) where
   compare (Path x) (Path y) = compare x y
 
--- | Same as 'Path.toFilePath'.
+-- | Same as 'show . Path.toFilePath'.
 --
 -- The following property holds:
 --
@@ -57,3 +67,20 @@ instance ToJSON (Path b t) where
   toEncoding (Path x) = toEncoding x
   {-# INLINE toEncoding #-}
 #endif
+
+instance Hashable (Path b t) where
+  hashWithSalt n (Path path) = hashWithSalt n path
+
+-- | Helper function: check if the filepath has any parent directories in it.
+-- This handles the logic of checking for different path separators on Windows.
+hasParentDir :: FilePath -> Bool
+hasParentDir filepath' =
+     (filepath' == "..") ||
+     ("/.." `isSuffixOf` filepath) ||
+     ("/../" `isInfixOf` filepath) ||
+     ("../" `isPrefixOf` filepath)
+  where
+    filepath =
+        case FilePath.pathSeparator of
+            '/' -> filepath'
+            x   -> map (\y -> if x == y then '/' else y) filepath'
